59a0a3844c
PR Test AMD / cancel-on-close (push) Has been skipped
PR Test NVIDIA ARM / scan (push) Has been skipped
PR Test NVIDIA / cancel-on-close (push) Has been skipped
PR Test AMD / scan (push) Has been skipped
PR Test NVIDIA ARM / cancel-on-close (push) Has been skipped
PR Test NVIDIA / scan (push) Has been skipped
Release Docker Images / build (cu129-torch-2.11.0) (push) Has been skipped
Release Docker Images / build (cu130-torch-2.11.0) (push) Has been skipped
Release PyPI / publish (push) Has been skipped
Scheduler Python Test / test (push) Successful in 27m19s
Docs / build (push) Successful in 28m8s
Scheduler C++ Test / test (push) Successful in 28m19s
Scheduler C++ Test / test-flat (push) Successful in 28m18s
Docs / deploy (push) Has been cancelled
PR Test AMD / finish (push) Has been cancelled
PR Test NVIDIA / finish (push) Has been cancelled
PR Test NVIDIA ARM / finish (push) Has been cancelled
PR Test NVIDIA ARM / ${{ matrix.name }} (${{ matrix.runner }}) (push) Has been cancelled
PR Test AMD / ${{ matrix.name }} (${{ matrix.runner }}) (push) Has been cancelled
PR Test NVIDIA / ${{ matrix.name }} (${{ matrix.runner }}) (push) Has been cancelled
82 lines
2.8 KiB
Python
82 lines
2.8 KiB
Python
# SPDX-License-Identifier: MIT AND Apache-2.0
|
|
# SPDX-FileCopyrightText: Copyright (c) 2026 LightSeek Foundation
|
|
# SPDX-FileCopyrightText: Copyright contributors to the FluentLLM project
|
|
#
|
|
# Copyright (c) 2026 LightSeek Foundation
|
|
#
|
|
# Permission is hereby granted, free of charge, to any person obtaining a copy
|
|
# of this software and associated documentation files (the "Software"), to deal
|
|
# in the Software without restriction, including without limitation the rights
|
|
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
|
|
# copies of the Software, and to permit persons to whom the Software is
|
|
# furnished to do so, subject to the following conditions:
|
|
#
|
|
# The above copyright notice and this permission notice shall be included in
|
|
# all copies or substantial portions of the Software.
|
|
#
|
|
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
|
|
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
|
|
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
|
|
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
|
|
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
|
|
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
|
|
# SOFTWARE.
|
|
|
|
from __future__ import annotations
|
|
|
|
import torch
|
|
import triton
|
|
import triton.language as tl
|
|
|
|
|
|
def move_kv_cache_native(
|
|
k_buffer: list[torch.Tensor],
|
|
v_buffer: list[torch.Tensor],
|
|
tgt_loc: torch.Tensor,
|
|
src_loc: torch.Tensor,
|
|
):
|
|
if tgt_loc.numel() == 0:
|
|
return
|
|
|
|
tgt_loc_flat = tgt_loc.view(-1).long()
|
|
src_loc_flat = src_loc.view(-1).long()
|
|
for k_cache, v_cache in zip(k_buffer, v_buffer):
|
|
k_cache[tgt_loc_flat] = k_cache[src_loc_flat]
|
|
v_cache[tgt_loc_flat] = v_cache[src_loc_flat]
|
|
|
|
|
|
@triton.jit
|
|
def copy_all_layer_kv_cache_tiled(
|
|
data_ptrs,
|
|
strides,
|
|
tgt_loc_ptr,
|
|
src_loc_ptr,
|
|
num_locs,
|
|
num_locs_upper: tl.constexpr,
|
|
BYTES_PER_TILE: tl.constexpr,
|
|
):
|
|
"""2D tiled kernel. Safe for in-place copy."""
|
|
bid = tl.program_id(0)
|
|
tid = tl.program_id(1)
|
|
|
|
stride = tl.load(strides + bid)
|
|
base_ptr = tl.load(data_ptrs + bid)
|
|
base_ptr = tl.cast(base_ptr, tl.pointer_type(tl.uint8))
|
|
|
|
byte_off = tid * BYTES_PER_TILE + tl.arange(0, BYTES_PER_TILE)
|
|
mask_byte = byte_off < stride
|
|
tl.multiple_of(byte_off, 16)
|
|
|
|
loc_idx = tl.arange(0, num_locs_upper)
|
|
mask_loc = loc_idx < num_locs
|
|
|
|
src = tl.load(src_loc_ptr + loc_idx, mask=mask_loc, other=0)
|
|
tgt = tl.load(tgt_loc_ptr + loc_idx, mask=mask_loc, other=0)
|
|
|
|
src_ptr = base_ptr + src[:, None] * stride + byte_off[None, :]
|
|
tgt_ptr = base_ptr + tgt[:, None] * stride + byte_off[None, :]
|
|
|
|
mask = mask_loc[:, None] & mask_byte[None, :]
|
|
vals = tl.load(src_ptr, mask=mask)
|
|
tl.store(tgt_ptr, vals, mask=mask)
|